@@ -1,8 +1,8 @@
 /*
  *      prefs.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
- *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
+ *      Copyright 2005-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: prefs.c 1928 2007-10-03 11:21:08Z ntrel $
+ * $Id: prefs.c 2186 2008-01-23 14:12:08Z eht16 $
  */
 
 /*
@@ -47,6 +47,8 @@
 #include "editor.h"
 #include "main.h"
 #include "treeviews.h"
+#include "printing.h"
+#include "geanywraplabel.h"
 
 #ifdef HAVE_VTE
 # include "vte.h"
@@ -75,8 +77,78 @@ static gboolean find_duplicate(guint idx, guint key, GdkModifierType mods, const
 static void on_toolbar_show_toggled(GtkToggleButton *togglebutton, gpointer user_data);
 static void on_show_notebook_tabs_toggled(GtkToggleButton *togglebutton, gpointer user_data);
 static void on_use_folding_toggled(GtkToggleButton *togglebutton, gpointer user_data);
-static void on_symbol_auto_completion_toggled(GtkToggleButton *togglebutton, gpointer user_data);
 static void on_open_encoding_toggled(GtkToggleButton *togglebutton, gpointer user_data);
+static void on_openfiles_visible_toggled(GtkToggleButton *togglebutton, gpointer user_data);
+static void on_prefs_print_radio_button_toggled(GtkToggleButton *togglebutton, gpointer user_data);
+static void on_prefs_print_page_header_toggled(GtkToggleButton *togglebutton, gpointer user_data);
+
+
+enum
+{
+	KB_TREE_ACTION,
+	KB_TREE_SHORTCUT,
+	KB_TREE_INDEX,
+};
+
+static void init_kb_tree()
+{
+	GtkCellRenderer *renderer;
+	GtkTreeViewColumn *column;
+
+	tree = GTK_TREE_VIEW(lookup_widget(ui_widgets.prefs_dialog, "treeview7"));
+	//g_object_set(tree, "vertical-separator", 6, NULL);
+
+	store = gtk_tree_store_new(3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT);
+	gtk_tree_view_set_model(GTK_TREE_VIEW(tree), GTK_TREE_MODEL(store));
+
+	renderer = gtk_cell_renderer_text_new();
+	column = gtk_tree_view_column_new_with_attributes(_("Action"), renderer, "text", KB_TREE_ACTION, NULL);
+	gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);
+
+	renderer = gtk_cell_renderer_text_new();
+	g_object_set(renderer, "editable", TRUE, NULL);
+	column = gtk_tree_view_column_new_with_attributes(_("Shortcut"), renderer, "text", KB_TREE_SHORTCUT, NULL);
+	gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);
+
+	// set policy settings for the scollwedwindow around the treeview again, because glade
+	// doesn't keep the settings
+	gtk_scrolled_window_set_policy(
+			GTK_SCROLLED_WINDOW(lookup_widget(ui_widgets.prefs_dialog, "scrolledwindow8")),
+			GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+
+	g_signal_connect(G_OBJECT(renderer), "edited", G_CALLBACK(on_cell_edited), NULL);
+	g_signal_connect(G_OBJECT(tree), "button-press-event",
+				G_CALLBACK(on_tree_view_button_press_event), NULL);
+	g_signal_connect(G_OBJECT(lookup_widget(ui_widgets.prefs_dialog, "button2")), "clicked",
+				G_CALLBACK(on_tree_view_button_press_event), NULL);
+}
+
+
+static void init_keybindings()
+{
+	GtkTreeIter parent, iter;
+	gint i;
+	gchar *key_string;
+
+	if (store == NULL)
+		init_kb_tree();
+
+	for (i = 0; i < GEANY_MAX_KEYS; i++)
+	{
+		if (keys[i]->section != NULL)
+		{
+			gtk_tree_store_append(store, &parent, NULL);
+			gtk_tree_store_set(store, &parent, KB_TREE_ACTION, keys[i]->section, -1);
+		}
+
+		key_string = gtk_accelerator_name(keys[i]->key, keys[i]->mods);
+		gtk_tree_store_append(store, &iter, &parent);
+		gtk_tree_store_set(store, &iter, KB_TREE_ACTION, keys[i]->label,
+			KB_TREE_SHORTCUT, key_string, KB_TREE_INDEX, i, -1);
+		g_free(key_string);
+	}
+	gtk_tree_view_expand_all(GTK_TREE_VIEW(tree));
+}
 
 
 void prefs_init_dialog(void)
@@ -92,6 +164,9 @@ void prefs_init_dialog(void)
 	widget = lookup_widget(ui_widgets.prefs_dialog, "check_load_session");
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), prefs.load_session);
 
+	widget = lookup_widget(ui_widgets.prefs_dialog, "check_project_session");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), project_prefs.project_session);
+
 	widget = lookup_widget(ui_widgets.prefs_dialog, "check_save_win_pos");
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), prefs.save_winpos);
 
@@ -132,6 +207,10 @@ void prefs_init_dialog(void)
 
 	widget = lookup_widget(ui_widgets.prefs_dialog, "check_list_openfiles");
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), prefs.sidebar_openfiles_visible);
+	on_openfiles_visible_toggled(GTK_TOGGLE_BUTTON(widget), NULL);
+
+	widget = lookup_widget(ui_widgets.prefs_dialog, "check_list_openfiles_fullpath");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), prefs.sidebar_openfiles_fullpath);
 
 	widget = lookup_widget(ui_widgets.prefs_dialog, "tagbar_font");
 	gtk_font_button_set_font_name(GTK_FONT_BUTTON(widget), prefs.tagbar_font);
@@ -197,6 +276,9 @@ void prefs_init_dialog(void)
 	widget = lookup_widget(ui_widgets.prefs_dialog, "check_toolbar_zoom");
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), prefs.toolbar_show_zoom);
 
+	widget = lookup_widget(ui_widgets.prefs_dialog, "check_toolbar_indent");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), prefs.toolbar_show_indent);
+
 	widget = lookup_widget(ui_widgets.prefs_dialog, "check_toolbar_undo");
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), prefs.toolbar_show_undo);
 
@@ -284,11 +366,14 @@ void prefs_init_dialog(void)
 	widget = lookup_widget(ui_widgets.prefs_dialog, "combo_auto_indent_mode");
 	gtk_combo_box_set_active(GTK_COMBO_BOX(widget), editor_prefs.indent_mode);
 
+	widget = lookup_widget(ui_widgets.prefs_dialog, "check_detect_indent");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), editor_prefs.detect_tab_mode);
+
 	widget = lookup_widget(ui_widgets.prefs_dialog, "check_line_wrapping");
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), editor_prefs.line_wrapping);
 
-	widget = lookup_widget(ui_widgets.prefs_dialog, "check_auto_complete");
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), editor_prefs.auto_complete_constructs);
+	widget = lookup_widget(ui_widgets.prefs_dialog, "check_complete_snippets");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), editor_prefs.complete_snippets);
 
 	widget = lookup_widget(ui_widgets.prefs_dialog, "check_xmltag");
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), editor_prefs.auto_close_xml_tags);
@@ -307,6 +392,9 @@ void prefs_init_dialog(void)
 	widget = lookup_widget(ui_widgets.prefs_dialog, "check_smart_home");
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), editor_prefs.smart_home_key);
 
+	widget = lookup_widget(ui_widgets.prefs_dialog, "check_newline_strip");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), editor_prefs.newline_strip);
+
 	if (editor_prefs.use_tabs)
 		widget = lookup_widget(ui_widgets.prefs_dialog, "radio_indent_tabs");
 	else
@@ -318,31 +406,24 @@ void prefs_init_dialog(void)
 
 	widget = lookup_widget(ui_widgets.prefs_dialog, "check_symbol_auto_completion");
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), editor_prefs.auto_complete_symbols);
-	on_symbol_auto_completion_toggled(GTK_TOGGLE_BUTTON(widget), NULL);
 
-	widget = lookup_widget(ui_widgets.prefs_dialog, "spin_autocheight");
-	gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), editor_prefs.autocompletion_max_height);
+	widget = lookup_widget(ui_widgets.prefs_dialog, "spin_symbollistheight");
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), editor_prefs.symbolcompletion_max_height);
+
+	widget = lookup_widget(ui_widgets.prefs_dialog, "spin_symbol_complete_chars");
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), editor_prefs.symbolcompletion_min_chars);
 
 
 	// Tools Settings
-#ifdef G_OS_WIN32
-        // hide related Make path setting
-        gtk_widget_set_sensitive(lookup_widget(ui_widgets.prefs_dialog, "label11"), FALSE);
-        gtk_widget_set_sensitive(lookup_widget(ui_widgets.prefs_dialog, "entry_com_make"), FALSE);
-        gtk_widget_set_sensitive(lookup_widget(ui_widgets.prefs_dialog, "button_make"), FALSE);
-#else
-        if (prefs.tools_make_cmd)
-                gtk_entry_set_text(GTK_ENTRY(lookup_widget(ui_widgets.prefs_dialog, "entry_com_make")), prefs.tools_make_cmd);
-#endif
+	if (prefs.tools_make_cmd)
+			gtk_entry_set_text(GTK_ENTRY(lookup_widget(ui_widgets.prefs_dialog, "entry_com_make")), prefs.tools_make_cmd);
+
     if (prefs.tools_term_cmd)
             gtk_entry_set_text(GTK_ENTRY(lookup_widget(ui_widgets.prefs_dialog, "entry_com_term")), prefs.tools_term_cmd);
 
 	if (prefs.tools_browser_cmd)
 		gtk_entry_set_text(GTK_ENTRY(lookup_widget(ui_widgets.prefs_dialog, "entry_browser")), prefs.tools_browser_cmd);
 
-	if (prefs.tools_print_cmd)
-		gtk_entry_set_text(GTK_ENTRY(lookup_widget(ui_widgets.prefs_dialog, "entry_print")), prefs.tools_print_cmd);
-
 	if (prefs.tools_grep_cmd)
 		gtk_entry_set_text(GTK_ENTRY(lookup_widget(ui_widgets.prefs_dialog, "entry_grep")), prefs.tools_grep_cmd);
 
@@ -365,59 +446,50 @@ void prefs_init_dialog(void)
 
 
 	// Keybindings
-	if (store == NULL)
+	init_keybindings();
+
+	// Printing
 	{
-		GtkCellRenderer *renderer;
-		GtkTreeViewColumn *column;
+		GtkWidget *widget_gtk = lookup_widget(ui_widgets.prefs_dialog, "radio_print_gtk");
+		if (printing_prefs.use_gtk_printing)
+			widget = widget_gtk;
+		else
+			widget = lookup_widget(ui_widgets.prefs_dialog, "radio_print_external");
 
-		tree = GTK_TREE_VIEW(lookup_widget(ui_widgets.prefs_dialog, "treeview7"));
-		//g_object_set(tree, "vertical-separator", 6, NULL);
-
-		store = gtk_tree_store_new(3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT);
-		gtk_tree_view_set_model(GTK_TREE_VIEW(tree), GTK_TREE_MODEL(store));
-
-		renderer = gtk_cell_renderer_text_new();
-		column = gtk_tree_view_column_new_with_attributes(_("Action"), renderer, "text", 0, NULL);
-		gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);
-
-		renderer = gtk_cell_renderer_text_new();
-		g_object_set(renderer, "editable", TRUE, NULL);
-		column = gtk_tree_view_column_new_with_attributes(_("Shortcut"), renderer, "text", 1, NULL);
-		gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);
-
-		// set policy settings for the scollwedwindow around the treeview again, because glade
-		// doesn't keep the settings
-		gtk_scrolled_window_set_policy(
-				GTK_SCROLLED_WINDOW(lookup_widget(ui_widgets.prefs_dialog, "scrolledwindow8")),
-				GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-
-		g_signal_connect(G_OBJECT(renderer), "edited", G_CALLBACK(on_cell_edited), NULL);
-		g_signal_connect(G_OBJECT(tree), "button-press-event",
-					G_CALLBACK(on_tree_view_button_press_event), NULL);
-		g_signal_connect(G_OBJECT(lookup_widget(ui_widgets.prefs_dialog, "button2")), "clicked",
-					G_CALLBACK(on_tree_view_button_press_event), NULL);
-	}
-
-	{
-		GtkTreeIter parent, iter;
-		gint i;
-		gchar *key_string;
-
-		for (i = 0; i < GEANY_MAX_KEYS; i++)
+#if GTK_CHECK_VERSION(2, 10, 0)
+		if (gtk_check_version(2, 10, 0) != NULL)
+#endif
 		{
-			if (keys[i]->section != NULL)
-			{
-				gtk_tree_store_append(store, &parent, NULL);
-				gtk_tree_store_set(store, &parent, 0, keys[i]->section, -1);
-			}
-
-			key_string = gtk_accelerator_name(keys[i]->key, keys[i]->mods);
-			gtk_tree_store_append(store, &iter, &parent);
-			gtk_tree_store_set(store, &iter, 0, keys[i]->label, 1, key_string, 2, i, -1);
-			g_free(key_string);
+			gtk_widget_set_sensitive(widget_gtk, FALSE); // disable the whole option block
+			widget = lookup_widget(ui_widgets.prefs_dialog, "radio_print_external");
 		}
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), TRUE);
+
+		on_prefs_print_radio_button_toggled(GTK_TOGGLE_BUTTON(widget_gtk), NULL);
 	}
-	gtk_tree_view_expand_all(GTK_TREE_VIEW(tree));
+	if (printing_prefs.external_print_cmd)
+		gtk_entry_set_text(
+			GTK_ENTRY(lookup_widget(ui_widgets.prefs_dialog, "entry_print_external_cmd")),
+			printing_prefs.external_print_cmd);
+
+	widget = lookup_widget(ui_widgets.prefs_dialog, "check_print_linenumbers");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), printing_prefs.print_line_numbers);
+
+	widget = lookup_widget(ui_widgets.prefs_dialog, "check_print_pagenumbers");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), printing_prefs.print_page_numbers);
+
+	widget = lookup_widget(ui_widgets.prefs_dialog, "check_print_pageheader");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), printing_prefs.print_page_header);
+	on_prefs_print_page_header_toggled(GTK_TOGGLE_BUTTON(widget), NULL);
+
+	widget = lookup_widget(ui_widgets.prefs_dialog, "check_print_basename");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), printing_prefs.page_header_basename);
+
+	if (printing_prefs.page_header_datefmt)
+		gtk_entry_set_text(
+			GTK_ENTRY(lookup_widget(ui_widgets.prefs_dialog, "entry_print_dateformat")),
+			printing_prefs.page_header_datefmt);
+
 
 #ifndef HAVE_PLUGINS
 	gtk_widget_set_sensitive(lookup_widget(ui_widgets.prefs_dialog, "check_plugins"), FALSE);
@@ -456,6 +528,9 @@ void prefs_init_dialog(void)
 		widget = lookup_widget(ui_widgets.prefs_dialog, "check_scroll_out");
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), vc->scroll_on_out);
 
+		widget = lookup_widget(ui_widgets.prefs_dialog, "check_enable_bash_keys");
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), vc->enable_bash_keys);
+
 		widget = lookup_widget(ui_widgets.prefs_dialog, "check_ignore_menu_key");
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), vc->ignore_menu_bar_accel);
 
@@ -464,6 +539,9 @@ void prefs_init_dialog(void)
 
 		widget = lookup_widget(ui_widgets.prefs_dialog, "check_run_in_vte");
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), vc->run_in_vte);
+
+		widget = lookup_widget(ui_widgets.prefs_dialog, "check_skip_script");
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), vc->skip_run_script);
 	}
 #endif
 }
@@ -488,6 +566,9 @@ on_prefs_button_clicked(GtkDialog *dialog, gint response, gpointer user_data)
 		widget = lookup_widget(ui_widgets.prefs_dialog, "check_load_session");
 		prefs.load_session = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
+		widget = lookup_widget(ui_widgets.prefs_dialog, "check_project_session");
+		project_prefs.project_session = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+
 		widget = lookup_widget(ui_widgets.prefs_dialog, "check_save_win_pos");
 		prefs.save_winpos = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
@@ -531,6 +612,9 @@ on_prefs_button_clicked(GtkDialog *dialog, gint response, gpointer user_data)
 		widget = lookup_widget(ui_widgets.prefs_dialog, "check_list_openfiles");
 		prefs.sidebar_openfiles_visible = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
+		widget = lookup_widget(ui_widgets.prefs_dialog, "check_list_openfiles_fullpath");
+		prefs.sidebar_openfiles_fullpath = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+
 		widget = lookup_widget(ui_widgets.prefs_dialog, "radio_long_line_line");
 		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) editor_prefs.long_line_type = 0;
 		else
@@ -576,6 +660,9 @@ on_prefs_button_clicked(GtkDialog *dialog, gint response, gpointer user_data)
 		widget = lookup_widget(ui_widgets.prefs_dialog, "check_toolbar_zoom");
 		prefs.toolbar_show_zoom = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
+		widget = lookup_widget(ui_widgets.prefs_dialog, "check_toolbar_indent");
+		prefs.toolbar_show_indent = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+
 		widget = lookup_widget(ui_widgets.prefs_dialog, "check_toolbar_undo");
 		prefs.toolbar_show_undo = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
@@ -669,8 +756,8 @@ on_prefs_button_clicked(GtkDialog *dialog, gint response, gpointer user_data)
 		widget = lookup_widget(ui_widgets.prefs_dialog, "check_line_wrapping");
 		editor_prefs.line_wrapping = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
-		widget = lookup_widget(ui_widgets.prefs_dialog, "check_auto_complete");
-		editor_prefs.auto_complete_constructs = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+		widget = lookup_widget(ui_widgets.prefs_dialog, "check_complete_snippets");
+		editor_prefs.complete_snippets = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
 		widget = lookup_widget(ui_widgets.prefs_dialog, "check_xmltag");
 		editor_prefs.auto_close_xml_tags = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
@@ -684,14 +771,36 @@ on_prefs_button_clicked(GtkDialog *dialog, gint response, gpointer user_data)
 		widget = lookup_widget(ui_widgets.prefs_dialog, "check_smart_home");
 		editor_prefs.smart_home_key = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
+		widget = lookup_widget(ui_widgets.prefs_dialog, "check_newline_strip");
+		editor_prefs.newline_strip = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+
 		widget = lookup_widget(ui_widgets.prefs_dialog, "radio_indent_tabs");
-		editor_prefs.use_tabs = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+		{
+			gboolean use_tabs = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+
+			// override each document setting only if the default has changed
+			if (editor_prefs.use_tabs != use_tabs)
+			{
+				editor_prefs.use_tabs = use_tabs;
+				for (i = 0; i < doc_array->len; i++)
+				{
+					if (doc_list[i].is_valid)
+						document_set_use_tabs(i, editor_prefs.use_tabs);
+				}
+			}
+		}
+
+		widget = lookup_widget(ui_widgets.prefs_dialog, "check_detect_indent");
+		editor_prefs.detect_tab_mode = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
 		widget = lookup_widget(ui_widgets.prefs_dialog, "check_symbol_auto_completion");
 		editor_prefs.auto_complete_symbols = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
-		widget = lookup_widget(ui_widgets.prefs_dialog, "spin_autocheight");
-		editor_prefs.autocompletion_max_height = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
+		widget = lookup_widget(ui_widgets.prefs_dialog, "spin_symbol_complete_chars");
+		editor_prefs.symbolcompletion_min_chars = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
+
+		widget = lookup_widget(ui_widgets.prefs_dialog, "spin_symbollistheight");
+		editor_prefs.symbolcompletion_max_height = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
 
 
 		// Tools Settings
@@ -707,10 +816,6 @@ on_prefs_button_clicked(GtkDialog *dialog, gint response, gpointer user_data)
 		g_free(prefs.tools_browser_cmd);
 		prefs.tools_browser_cmd = g_strdup(gtk_entry_get_text(GTK_ENTRY(widget)));
 
-		widget = lookup_widget(ui_widgets.prefs_dialog, "entry_print");
-		g_free(prefs.tools_print_cmd);
-		prefs.tools_print_cmd = g_strdup(gtk_entry_get_text(GTK_ENTRY(widget)));
-
 		widget = lookup_widget(ui_widgets.prefs_dialog, "entry_grep");
 		g_free(prefs.tools_grep_cmd);
 		prefs.tools_grep_cmd = g_strdup(gtk_entry_get_text(GTK_ENTRY(widget)));
@@ -741,6 +846,32 @@ on_prefs_button_clicked(GtkDialog *dialog, gint response, gpointer user_data)
 		// Keybindings
 		if (edited) keybindings_write_to_file();
 
+
+		// Printing
+		widget = lookup_widget(ui_widgets.prefs_dialog, "radio_print_gtk");
+		printing_prefs.use_gtk_printing = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+
+		widget = lookup_widget(ui_widgets.prefs_dialog, "entry_print_external_cmd");
+		g_free(printing_prefs.external_print_cmd);
+		printing_prefs.external_print_cmd = g_strdup(gtk_entry_get_text(GTK_ENTRY(widget)));
+
+		widget = lookup_widget(ui_widgets.prefs_dialog, "check_print_linenumbers");
+		printing_prefs.print_line_numbers = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+
+		widget = lookup_widget(ui_widgets.prefs_dialog, "check_print_pagenumbers");
+		printing_prefs.print_page_numbers = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+
+		widget = lookup_widget(ui_widgets.prefs_dialog, "check_print_pageheader");
+		printing_prefs.print_page_header = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+
+		widget = lookup_widget(ui_widgets.prefs_dialog, "check_print_basename");
+		printing_prefs.page_header_basename = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+
+		widget = lookup_widget(ui_widgets.prefs_dialog, "entry_print_dateformat");
+		g_free(printing_prefs.page_header_datefmt);
+		printing_prefs.page_header_datefmt = g_strdup(gtk_entry_get_text(GTK_ENTRY(widget)));
+
+
 #ifdef HAVE_VTE
 		widget = lookup_widget(ui_widgets.prefs_dialog, "check_vte");
 		vte_info.load_vte = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
@@ -765,6 +896,9 @@ on_prefs_button_clicked(GtkDialog *dialog, gint response, gpointer user_data)
 			widget = lookup_widget(ui_widgets.prefs_dialog, "check_scroll_out");
 			vc->scroll_on_out = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
+			widget = lookup_widget(ui_widgets.prefs_dialog, "check_enable_bash_keys");
+			vc->enable_bash_keys = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+
 			widget = lookup_widget(ui_widgets.prefs_dialog, "check_ignore_menu_key");
 			vc->ignore_menu_bar_accel = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
@@ -774,16 +908,20 @@ on_prefs_button_clicked(GtkDialog *dialog, gint response, gpointer user_data)
 			widget = lookup_widget(ui_widgets.prefs_dialog, "check_run_in_vte");
 			vc->run_in_vte = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
+			widget = lookup_widget(ui_widgets.prefs_dialog, "check_skip_script");
+			vc->skip_run_script = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+
 			vte_apply_user_settings();
 		}
 #endif
 
 		// apply the changes made
 		ui_statusbar_showhide(prefs.statusbar_visible);
+		treeviews_openfiles_update_all(); // to update if full path setting has changed
 		ui_update_toolbar_items();
 		ui_update_toolbar_icons(prefs.toolbar_icon_size);
 		gtk_toolbar_set_style(GTK_TOOLBAR(app->toolbar), prefs.toolbar_icon_style);
-		ui_treeviews_show_hide(FALSE);
+		ui_sidebar_show_hide();
 		gtk_notebook_set_show_tabs(GTK_NOTEBOOK(app->notebook), prefs.show_notebook_tabs);
 
 		gtk_notebook_set_tab_pos(GTK_NOTEBOOK(app->notebook), prefs.tab_pos_editor);
@@ -1050,6 +1188,34 @@ static void on_dialog_response(GtkWidget *dialog, gint response, gpointer iter)
 }
 
 
+static gboolean find_iter(guint i, GtkTreeIter *iter)
+{
+	GtkTreeModel *model = GTK_TREE_MODEL(store);
+	guint idx;
+	GtkTreeIter parent;
+
+	if (! gtk_tree_model_get_iter_first(model, &parent))
+		return FALSE;	// no items
+
+	while (TRUE)
+	{
+		if (! gtk_tree_model_iter_children(model, iter, &parent))
+			return FALSE;
+
+		while (TRUE)
+		{
+			gtk_tree_model_get(model, iter, 2, &idx, -1);
+			if (idx == i)
+				return TRUE;
+			if (! gtk_tree_model_iter_next(model, iter))
+				break;
+		}
+		if (! gtk_tree_model_iter_next(model, &parent))
+			return FALSE;
+	}
+}
+
+
 // test if the entered key combination is already used
 static gboolean find_duplicate(guint idx, guint key, GdkModifierType mods, const gchar *action)
 {
@@ -1064,9 +1230,19 @@ static gboolean find_duplicate(guint idx, guint key, GdkModifierType mods, const
 		if (keys[i]->key == key && keys[i]->mods == mods
 			&& ! (keys[i]->key == keys[idx]->key && keys[i]->mods == keys[idx]->mods))
 		{
-			dialogs_show_msgbox(GTK_MESSAGE_ERROR,
-				_("The combination '%s' is already used for \"%s\". Please choose another one."),
-				action, keys[i]->label);
+			if (dialogs_show_question_full(app->window, _("_Override"), GTK_STOCK_CANCEL,
+				_("Override that keybinding?"),
+				_("The combination '%s' is already used for \"%s\"."),
+				action, keys[i]->label))
+			{
+				GtkTreeIter iter;
+
+				keys[i]->key = 0;
+				keys[i]->mods = 0;
+				if (find_iter(i, &iter))
+					gtk_tree_store_set(store, &iter, 1, NULL, -1);	// clear item
+				continue;
+			}
 			return TRUE;
 		}
 	}
@@ -1101,14 +1277,6 @@ static void on_use_folding_toggled(GtkToggleButton *togglebutton, gpointer user_
 }
 
 
-static void on_symbol_auto_completion_toggled(GtkToggleButton *togglebutton, gpointer user_data)
-{
-	gboolean sens = gtk_toggle_button_get_active(togglebutton);
-
-	gtk_widget_set_sensitive(lookup_widget(ui_widgets.prefs_dialog, "hbox6"), sens);
-}
-
-
 static void on_open_encoding_toggled(GtkToggleButton *togglebutton, gpointer user_data)
 {
 	gboolean sens = gtk_toggle_button_get_active(togglebutton);
@@ -1118,11 +1286,38 @@ static void on_open_encoding_toggled(GtkToggleButton *togglebutton, gpointer use
 }
 
 
+static void on_openfiles_visible_toggled(GtkToggleButton *togglebutton, gpointer user_data)
+{
+	gboolean sens = gtk_toggle_button_get_active(togglebutton);
+
+	gtk_widget_set_sensitive(lookup_widget(ui_widgets.prefs_dialog, "check_list_openfiles_fullpath"), sens);
+}
+
+
+static void on_prefs_print_radio_button_toggled(GtkToggleButton *togglebutton, gpointer user_data)
+{
+	gboolean sens = gtk_toggle_button_get_active(togglebutton);
+
+	gtk_widget_set_sensitive(lookup_widget(ui_widgets.prefs_dialog, "vbox29"), sens);
+	gtk_widget_set_sensitive(lookup_widget(ui_widgets.prefs_dialog, "hbox9"), ! sens);
+}
+
+
+static void on_prefs_print_page_header_toggled(GtkToggleButton *togglebutton, gpointer user_data)
+{
+	gboolean sens = gtk_toggle_button_get_active(togglebutton);
+
+	gtk_widget_set_sensitive(lookup_widget(ui_widgets.prefs_dialog, "check_print_basename"), sens);
+	gtk_widget_set_sensitive(lookup_widget(ui_widgets.prefs_dialog, "entry_print_dateformat"), sens);
+}
+
+
 void prefs_show_dialog(void)
 {
 	if (ui_widgets.prefs_dialog == NULL)
 	{
 		GtkWidget *combo_new, *combo_open;
+		GtkWidget *label;
 		guint i;
 		gchar *encoding_string;
 
@@ -1143,6 +1338,35 @@ void prefs_show_dialog(void)
 			g_free(encoding_string);
 		}
 
+		// add manually GeanyWrapLabels because it can't be added with Glade
+		// page Tools
+		label = geany_wrap_label_new(_("Enter tool paths below. Tools you do not need can be left blank."));
+		gtk_widget_show(label);
+		gtk_box_pack_start(GTK_BOX(lookup_widget(ui_widgets.prefs_dialog, "vbox33")),
+			label, FALSE, TRUE, 5);
+		// page Templates
+		label = geany_wrap_label_new(_("Set the information to be used in templates. See the documentation for details."));
+		gtk_widget_show(label);
+		gtk_box_pack_start(GTK_BOX(lookup_widget(ui_widgets.prefs_dialog, "vbox31")),
+			label, FALSE, TRUE, 5);
+		label = geany_wrap_label_new(_("<i>Notice: For all changes you make here to take effect, you need to restart Geany.</i>"));
+		gtk_widget_show(label);
+		gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
+		gtk_box_pack_start(GTK_BOX(lookup_widget(ui_widgets.prefs_dialog, "vbox9")),
+			label, FALSE, TRUE, 5);
+		// page Keybindings
+		label = geany_wrap_label_new(_("Here you can change keyboard shortcuts for various actions. Select one and press the Change button to enter a new shortcut, or double click on an action to edit the string representation of the shortcut directly."));
+		gtk_widget_show(label);
+		gtk_box_pack_start(GTK_BOX(lookup_widget(ui_widgets.prefs_dialog, "vbox32")),
+			label, FALSE, TRUE, 5);
+		// page Printing
+		label = geany_wrap_label_new(_("<i>Notice: Native GTK printing is only available if Geany was built against GTK 2.10 (or above) <b>and</b> Geany is running with GTK 2.10 (or above).</i>"));
+		gtk_widget_show(label);
+		gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
+		gtk_box_pack_start(GTK_BOX(lookup_widget(ui_widgets.prefs_dialog, "vbox27")),
+			label, FALSE, TRUE, 5);
+
+
 #ifdef HAVE_VTE
 		vte_append_preferences_tab();
 #endif
@@ -1174,10 +1398,18 @@ void prefs_show_dialog(void)
 				"clicked", G_CALLBACK(on_prefs_tools_button_clicked), lookup_widget(ui_widgets.prefs_dialog, "entry_grep"));
 
 		// tools commands
-		g_signal_connect((gpointer) lookup_widget(ui_widgets.prefs_dialog, "button_print"),
-				"clicked", G_CALLBACK(on_prefs_tools_button_clicked), lookup_widget(ui_widgets.prefs_dialog, "entry_print"));
 		g_signal_connect((gpointer) lookup_widget(ui_widgets.prefs_dialog, "button_contextaction"),
-				"clicked", G_CALLBACK(on_prefs_tools_button_clicked), lookup_widget(ui_widgets.prefs_dialog, "entry_contextaction"));
+			"clicked", G_CALLBACK(on_prefs_tools_button_clicked), lookup_widget(ui_widgets.prefs_dialog, "entry_contextaction"));
+		g_signal_connect((gpointer) lookup_widget(ui_widgets.prefs_dialog, "button_contextaction"),
+			"clicked", G_CALLBACK(on_prefs_tools_button_clicked), lookup_widget(ui_widgets.prefs_dialog, "entry_contextaction"));
+
+		// printing
+		g_signal_connect((gpointer) lookup_widget(ui_widgets.prefs_dialog, "button_print_external_cmd"),
+			"clicked", G_CALLBACK(on_prefs_tools_button_clicked), lookup_widget(ui_widgets.prefs_dialog, "entry_print_external_cmd"));
+		g_signal_connect((gpointer) lookup_widget(ui_widgets.prefs_dialog, "radio_print_gtk"),
+			"toggled", G_CALLBACK(on_prefs_print_radio_button_toggled), NULL);
+		g_signal_connect((gpointer) lookup_widget(ui_widgets.prefs_dialog, "check_print_pageheader"),
+			"toggled", G_CALLBACK(on_prefs_print_page_header_toggled), NULL);
 
 		g_signal_connect((gpointer) lookup_widget(ui_widgets.prefs_dialog, "check_toolbar_show"),
 				"toggled", G_CALLBACK(on_toolbar_show_toggled), NULL);
@@ -1185,10 +1417,10 @@ void prefs_show_dialog(void)
 				"toggled", G_CALLBACK(on_show_notebook_tabs_toggled), NULL);
 		g_signal_connect((gpointer) lookup_widget(ui_widgets.prefs_dialog, "check_folding"),
 				"toggled", G_CALLBACK(on_use_folding_toggled), NULL);
-		g_signal_connect((gpointer) lookup_widget(ui_widgets.prefs_dialog, "check_symbol_auto_completion"),
-				"toggled", G_CALLBACK(on_symbol_auto_completion_toggled), NULL);
 		g_signal_connect((gpointer) lookup_widget(ui_widgets.prefs_dialog, "check_open_encoding"),
 				"toggled", G_CALLBACK(on_open_encoding_toggled), NULL);
+		g_signal_connect((gpointer) lookup_widget(ui_widgets.prefs_dialog, "check_list_openfiles"),
+				"toggled", G_CALLBACK(on_openfiles_visible_toggled), NULL);
 	}
 
 	prefs_init_dialog();
