@@ -1,7 +1,8 @@
 /*
  *      vte.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006 Enrico Troeger <enrico.troeger@uvena.de>
+ *      Copyright 2005-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2008 Nick Treleaven <<nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: vte.h 1078 2006-12-10 21:29:04Z eht16 $
+ * $Id: vte.h 2150 2008-01-06 18:11:57Z eht16 $
  */
 
 
@@ -53,6 +54,8 @@ typedef struct
 	gboolean ignore_menu_bar_accel;
 	gboolean follow_path;
 	gboolean run_in_vte;
+	gboolean skip_run_script;
+	gboolean enable_bash_keys;
 	gint scrollback_lines;
 	gchar *emulation;
 	gchar *shell;
@@ -69,7 +72,7 @@ void vte_close(void);
 
 void vte_apply_user_settings(void);
 
-void vte_send_cmd(const gchar *cmd);
+gboolean vte_send_cmd(const gchar *cmd);
 
 const gchar* vte_get_working_directory(void);
 
@@ -115,8 +118,6 @@ struct VteFunctions
 	void (*vte_terminal_set_word_chars) (VteTerminal *terminal, const char *spec);
 	void (*vte_terminal_set_mouse_autohide) (VteTerminal *terminal, gboolean setting);
 	void (*vte_terminal_reset) (VteTerminal *terminal, gboolean full, gboolean clear_history);
-	void (*vte_terminal_set_encoding) (VteTerminal *terminal, const char *codeset);
-	void (*vte_terminal_set_cursor_blinks) (VteTerminal *terminal, gboolean blink);
 	GtkType (*vte_terminal_get_type) (void);
 	void (*vte_terminal_set_scroll_on_output) (VteTerminal *terminal, gboolean scroll);
 	void (*vte_terminal_set_scroll_on_keystroke) (VteTerminal *terminal, gboolean scroll);
